#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::AgentUI;

=head1 NAME

SrSv::AgentUI - User interface for IRC agents

=cut

use strict;

use Carp;

use SrSv::User::Notice;
use SrSv::Help qw(send_help);

use SrSv::AgentUI::Guts qw(get_token_type is_token_optional get_optional_index);
use SrSv::AgentUI::Hint qw(get_syntax_hint);

our %uis;

our $flood_check;

our %unbounded_types = (
	TEXT => 1,
	LIST => 1,
);

sub check_prototype($) {
	my ($command) = @_;

	my $real = prototype($command->{CALL});
	my $expected = '$' x ($command->{ARGUMENT_COUNT} + 1); # one for $user

	if($real ne $expected) {
		croak "UI syntax does not match function prototype - was $real, expected $expected";
	}

	return undef;
}

sub is_unbounded($) {
	my ($command) = @_;

	my @al = @{$command->{ARGUMENT_LIST}};
	my $last = pop @al;

	return 0 unless $unbounded_types{get_token_type($last)};

	foreach my $arg (@al) { #all but the last one
		(not is_token_optional($arg))
			or croak "In an unbounded UI command, only the last argument may be optional";
		
		(not $unbounded_types{get_token_type($arg)})
			or croak "UI command cannot have two unbounded arguments";
	}

	return 1;
}

sub is_list($) {
	my ($command) = @_;

	my $last = @{$command->{ARGUMENT_LIST}}[-1];

	return (get_token_type($last) eq 'LIST');
}

sub send_syntax_hint($$$) {
	my ($user, $ui, $command) = @_;

	notice $user, "Syntax: $command->{SYNTAX_HINT}", "For more help, type: \002$ui->{PREFIX} help ".lc $command->{SYNONYMS}[0];
}

sub get_add_command_sub($) {
	my ($p) = @_;

	return sub ($) {
		add_command($p, @_);
	}
}

sub add_command {
	my ($ui, $command) = @_;

	my ($synonyms, @syntax) = split(/\s+/, $command->{SYNTAX});

	my @synonyms = split(m|/|, $synonyms);

	foreach my $s (@synonyms) {
		$s = uc $s;
		croak qq{UI command "$s" is already in use} if $ui->{COMMAND_INDEX}{$s};

		$ui->{COMMAND_INDEX}{$s} = $command;
	}
	
	push @{$ui->{COMMANDS}}, $command;
	
	$command->{SYNONYMS} = \@synonyms;
	$command->{ARGUMENT_LIST} = \@syntax;
	$command->{ARGUMENT_COUNT} = scalar(@syntax);
	$command->{OPTIONAL_ARGUMENT} = get_optional_index($command);
	$command->{UNBOUNDED} = is_unbounded($command);
	$command->{LIST} = is_list($command);
	$command->{SYNTAX_HINT} = get_syntax_hint($command);

	check_prototype($command);
}

sub get_dispatch_sub($) {
	my ($p) = @_;

	return sub ($$$) {
		dispatch($p, @_);
	}
}

sub dispatch {
	my ($ui, $src, $dst, $msg) = @_;
	my $user = { NICK => $src, AGENT => $dst };

	return if $flood_check->($user);

	my ($cmd, $args) = split(/\s+/, $msg, 2);

	$cmd = uc $cmd;
	my $command = $ui->{COMMAND_INDEX}{$cmd};

	unless(defined $command) {
		notice $user, qq{"$cmd" is not a $ui->{NAME} command.}, "For help, type: \002$ui->{PREFIX} help";
		return;
	}

	my @args;
	
	if($command->{UNBOUNDED}) {
		@args = split(/\s+/, $args, $command->{ARGUMENT_COUNT});
	} else {
		@args = split(/\s+/, $args);
	}

	if(@args != $command->{ARGUMENT_COUNT}) {
		if(defined($command->{OPTIONAL_ARGUMENT}) and @args == ($command->{ARGUMENT_COUNT})-1) {
			my $default;

			if($command->{OPTIONAL_DEFAULT_SRC}) {
				$default = $src;
			}
			
			splice(@args, $command->{OPTIONAL_ARGUMENT}, 0, $default);
		}
		else {
			send_syntax_hint($user, $ui, $command);
			return;
		}
	}

	if($command->{LIST}) {
		my $arg = @args-1;
		$args[$arg] = [split(/\s+/, $args[$arg])];
	}

	&{$command->{CALL}}($user, @args);
}

sub get_help_sub($) {
	my ($name) = @_;

	return sub ($$) {
		send_help($_[0], "$name $_[1]");
	}
}

sub import($%) {
	my (undef, $p) = @_;

	if($p->{FLOOD_CHECK_SUB}) {
		$flood_check = $p->{FLOOD_CHECK_SUB};
		return;
	}

	my $package = caller;

	$p->{COMMANDS} = [];
	$p->{COMMAND_INDEX} = {};

	add_command($p, { SYNTAX => 'help [TEXT:topic]', CALL => get_help_sub($p->{NAME})} );

	{
		no strict 'refs';
		*{"$package\::$p->{DISPATCH_SUB}"} = get_dispatch_sub($p);
		*{"$package\::$p->{ADD_COMMAND_SUB}"} = get_add_command_sub($p);
	}

	$uis{$p->{NAME}} = $p;
}

1;

__END__

=head1 SYNOPSIS

 use SrSv::AgentUI {
 	NAME => $myagent_nick,
 	PREFIX => '/msg myagent',
 	ADD_COMMAND_SUB => 'myagent_command',
 	DISPATCH_SUB => 'myagent_dispatch',
 };
 
 use SrSv::IRCd::Event qw(addhandler);
 addhandler('PRIVMSG', undef, $myagent_nick, 'myagent_dispatch');
 
 myagent_command {
 	SYNTAX => 'frobnicate/frob/fr [nick] password',
	OPTIONAL_DEFAULT_SRC => 1,
	CALL => \&myagent_frobnicate,
 };
 
 sub frobnicate($$$) {
 	my ($user, $nick, $password) = @_;
	
	...
 }

=head1 DESCRIPTION

This module creates DALnet-style user interfaces for IRC agents.  It exports
two subroutines to the caller's namespace; one adds commands to the interface,
the other handles incoming messages from users.

=head2 Adding a command

The subroutine defined with ADD_COMMAND_SUB takes a hash reference containing
the following keys:

=over

=item SYNTAX

A string that defines the syntax for the command.  It begins with the name
of the command, optionally with additional synonyms separated by slashes.
Next is a list of named parameters, separated by spaces.  The names of the
parameters are used only to generate help texts, they have no other effect.

To make a parameter optional, place square brackets around it.  Currently,
only one optional parameter is allowed.

To accept freeform text with spaces, prepend TEXT: to the parameter name.

To accept a list of parameters, prepend LIST: to the parameter name.  The
list will be passed to the callback as an array reference.

TEXT and LIST parameters are considered "unbounded" and must be the last
parameter of the command.  Their use is incompatible with optional
parameters, although they may be optional themselves.

=item OPTIONAL_DEFAULT_SRC

If this is set to a true value, the optional argument will automatically
be filled in to the nick of the user who sent the command.

=item CALL

A reference to the subroutine that will process the command.

=back

