#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickServ::Enforcer;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT_OK = qw(check_identify protect collide enforcer_quit %enforcers) }

use SrSv::Message qw(add_callback);

use SrSv::IRCd::Event qw(addhandler);
use SrSv::Timer qw(add_timer);

use SrSv::NickServ;
use SrSv::NickServ::Protect;

use SrSv::User qw(get_user_nick is_online);
use SrSv::User::Guest;

use SrSv::NickReg qw(is_registered);
use SrSv::NickReg::Protect qw(nr_protect_level);
use SrSv::NickReg::Id qw(is_identified);
use SrSv::NickReg::Flags;

use SrSv::Shared qw(%preenforce %enforcers);

addhandler('NICKCHANGE', undef, undef, 'nick_change');

sub nick_change($$$) {
	my ($old, $new, $time) = @_;

	return if(lc $old eq lc $new);

	if($preenforce{lc $old}) {
		ircd::agent_connect($old, 'enforcer', undef, '+i', 'NickServ Enforcement Agent');
		undef($preenforce{lc $old});
		$enforcers{lc $old} = 1;
		add_timer($old, 60, __PACKAGE__, 'enforcer_quit');
	}
}

add_callback {
	TRIGGER_COND => { CLASS => 'SrSv::User::Event', TYPE => 'NEW_USER'},
	CALL => __PACKAGE__ . '::user_event',
	AFTER => 'LogonNews',
};

sub user_event($$) {
	my ($message) = @_;

	check_identify($message->{USER});
}

sub check_identify($) {
	my ($user) = @_;
	my $nick = get_user_nick($user);
        if(is_registered($nick)) {
                if(is_identified($user, $nick)) {
                        ircd::setumode($nsnick, $nick, '+r');
                } else {
                        protect($nick);
                }
        }
}

sub protect($) {
	my ($nick) = @_;

	return if nr_flag($nick, NRF_FREEZE);
	my $lev = nr_protect_level($nick);
	
	ircd::notice($nsnick, $nick,
		"This nickname is registered and protected. If it is your",
		"nick, type \002/msg NickServ IDENTIFY <password>\002. Otherwise,",
		"please choose a different nick."
	) unless($lev==3);

	if($lev == 1) {
		warn_countdown("$nick 60");
	}
	elsif($lev==2) {
		collide($nick);
	}
	elsif($lev==3) {
		ircd::irckill($nsnick, $nick, "Unauthorized nick use with KILL protection enabled.");
		ircd::agent_connect($nick, 'enforcer', undef, '+i', 'NickServ Enforcement Agent');
		$enforcers{lc $nick} = 1;
		add_timer($nick, 60, __PACKAGE__, 'enforcer_quit');
	}
	
	return;
}

sub warn_countdown($) {
	my ($cookie)  = @_;
	my ($nick, $rem) = split(/ /, $cookie);
	my $user = { NICK => $nick };
	
	if (is_identified($user, $nick) or (!is_online($nick)) or (!is_registered($nick))) { return };

	if($rem == 0) {
		ircd::notice($nsnick, $nick,
			'Your nick is now being changed.');
		collide($nick);
	} else {
		ircd::notice($nsnick, $nick,
			"If you do not identify or change your nick in $rem seconds, your nick will be changed.");
		$rem -= 20;
		add_timer("$nick $rem", 20, __PACKAGE__, 'warn_countdown');
	}
}

sub collide($) {
	my ($nick) = @_;
	
	my $newnick = guestnick($nick);
	
	# We have to wait to do the enforcing until the nick actually changes
	$preenforce{lc $nick} = 1;
	add_timer($nick, 60, __PACKAGE__, 'enforcer_quit');
	return $newnick;
}

sub enforcer_quit($) {
	my ($nick) = @_;
	if($enforcers{lc $nick}) {
		undef($enforcers{lc $nick});
		ircd::agent_quit($nick, 'Enforcer Leaving');
		return 1;
	}
	return 0;
}

1;
