#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::NickServ::Id;

use strict;

use Exporter 'import';
BEGIN { our @EXPORT = qw(ns_identify ns_logout ns_release ns_ghost ns_recover) }

use SrSv::NickServ;
use SrSv::NickServ::UI;
use SrSv::NickServ::Enforcer qw(protect collide enforcer_quit);

use SrSv::Errors;

use SrSv::NickReg::Password;
use SrSv::NickReg::Flags;
use SrSv::NickReg::Id qw(nr_identify nr_logout is_identified);
use SrSv::NickReg::Alias qw(get_root_nick);

use SrSv::User qw(get_user_nick);
use SrSv::User::InvalidPass;
use SrSv::User::Notice;
use SrSv::Errors;

ns_command {
	SYNTAX => 'identify/id [nick] password',
	OPTIONAL_DEFAULT_SRC => 1,
	CALL => \&ns_identify,
};

sub ns_identify($$$) {
	my ($user, $nick, $pass) = @_;
	my $src = get_user_nick($user);

	if(is_identified($user, $nick)) {
		notice($user, 'You are already identified for nick '.$nick.'.');
		return 0;
	}

	my $root = get_root_nick($nick);
	unless($root) {
		notice($user, 'Your nick is not registered.');
		return 0;
	}

	if(nr_flag($root, NRF_FREEZE)) {
		notice($user, "This nick has been frozen and may not be used.", $err_deny);
		services::ulog($nsnick, log::INFO(), "attempted to identify to frozen nick $nick", $user);
		return;
	}

	unless(chk_pass($root, $pass, $user)) {
		if(inc_nick_inval($user)) {
			notice($user, $err_pass);
		}
		services::ulog($nsnick, log::INFO(), "failed to identify to nick $nick", $user);
		return 0;
	}

	nr_identify($user, $root);

	notice($user, 'You are now identified.');
	ircd::setumode($nsnick, $src, '+r') if lc($src) eq lc($nick);

	hostserv::hs_on($user, $root);

	nickserv::do_svssilence($user, $root);
	nickserv::do_svswatch($user, $root);

	chanserv::akick_alluser($user);
	chanserv::set_modes_allchan($user) unless nr_flag($nick, NRF_NEVEROP);
	
	services::ulog($nsnick, log::INFO(), "identified to nick $nick", $user);

	memoserv::notify($user, $root);

	if(enforcer_quit($nick)) {
		notice($user, 'Your nick has been released from custody.');
		return 2;
	}
	return 1;
}

ns_command {
	SYNTAX => 'logout',
	CALL => \&ns_logout,
};

sub ns_logout($) {
	my ($user) = @_;

	nr_logout($user);
	
	ircd::setumode($nsnick, get_user_nick($user), '-r');
	notice($user, 'You are now logged out.');
	services::ulog($nsnick, log::INFO(), "used NickServ LOGOUT", $user);
}

ns_command {
	SYNTAX => 'release nick [password]',
	CALL => \&ns_release,
};

sub ns_release($$$) {
	my ($user, $nick, $pass) = @_;

	if(nr_flag($nick, NRF_FREEZE)) {
		notice($user, "This nick has been frozen and may not be used.", $err_deny);
		return;
	}

	unless(is_identified($user, $nick)) {
		if($pass) {
			my $s = ns_identify($user, $nick, $pass);
			return if($s == 0); #failed to identify
			if($s == 1) {
				notice($user, "Nick $nick is not being held.");
				return;
			}
		} else {
			notice($user, $err_deny);
			return;
		}
	}

	if(enforcer_quit($nick)) {
		notice($user, 'Your nick has been released from custody.');
	} else {
		notice($user, "Nick $nick is not being held.");
	}
}

ns_command {
	SYNTAX => 'ghost nick [password]',
	CALL => \&ns_ghost,
};

sub ns_ghost($$$) {
	my ($user, $nick, $pass) = @_;
	my $src = get_user_nick($user);

	if(nr_flag($nick, NRF_FREEZE)) {
		notice($user, "This nick has been frozen and may not be used.", $err_deny);
		return;
	}

	unless(is_identified($user, $nick)) {
		if($pass) {
			my $s = ns_identify($user, $nick, $pass);
			return if($s == 0); #failed to identify
		} else {
			notice($user, $err_deny);
			return;
		}
	}

	if(!is_online($nick)) {
		notice($user, "\002$nick\002 is not online");
		return;
	} elsif(lc $src eq lc $nick) {
		notice($user, "I'm sorry, $src, I'm afraid I can't do that.");
		return;

	} else {
		ircd::irckill($nsnick, $nick, "GHOST command used by $src");
		notice($user, "Your ghost has been disconnected");
		services::ulog($nsnick, log::INFO(), "used NickServ GHOST on $nick", $user);
		#nick_delete($nick);
		return;
	}
}

ns_command {
	SYNTAX => 'recover nick [password]',
	CALL => \&ns_recover,
};

sub ns_recover($$$) {
	my ($user, $nick, $pass) = @_;
	my $src = get_user_nick($user);

	if(nr_flag($nick, NRF_FREEZE, +1)) {
		notice($user, "This nick has been frozen and may not be used.", $err_deny);
		return;
	}

	unless(is_identified($user, $nick)) {
		if($pass) {
			my $s = ns_identify($user, $nick, $pass);
			return if($s == 0); #failed to identify
		} else {
			notice($user, $err_deny);
			return;
		}
	}

	if(!is_online($nick)) {
		notice($user, "\002$nick\002 is not online");
		return;
	} elsif(lc $src eq lc $nick) {
		notice($user, "I'm sorry, $src, I'm afraid I can't do that.");
		return;

	} else {
		collide($nick);
		notice($user, "User claiming your nick has been collided", 
			"/msg NickServ RELEASE $nick to get it back before the one-minute timeout.");
		services::ulog($nsnick, log::INFO(), "used NickServ RECOVER on $nick", $user);
		return;
	}
}

1;
