#	This file is part of SurrealServices.
#
#	SurrealServices is free software; you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation; either version 2 of the License, or
#	(at your option) any later version.
#
#	SurrealServices is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with SurrealServices; if not, write to the Free Software
#	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package SrSv::User;

use strict;

use Exporter 'import';
BEGIN {
	my %flags = (
		UF_FINISHED => 1,
		UF_ONLINE => 2,
	);

	our @EXPORT = keys(%flags);
	our @EXPORT_OK = qw(get_user_id get_user_nick is_online chk_online $get_user_id);
	our %EXPORT_TAGS = (flags => [keys(%flags)]);
	
	require constant; import constant (\%flags);
}

use SrSv::User::Event;
use SrSv::User::Notice;
use SrSv::Process::Init;
use SrSv::MySQL '$dbh';
use SrSv::NickServ::Enforcer qw(%enforcers);
use SrSv::IRCd::State qw(synced);
use SrSv::Agent qw(is_agent);
use SrSv::Debug;

our ($get_user_id, $get_user_nick, $get_nickchg, $is_online);

proc_init {
	$get_user_id = $dbh->prepare("SELECT id FROM user WHERE nick=?");
	$get_user_nick = $dbh->prepare("SELECT nick FROM user WHERE id=?");
	$get_nickchg = $dbh->prepare("SELECT nickid FROM nickchg WHERE nick=?");
	$is_online = $dbh->prepare("SELECT 1 FROM user WHERE nick=? AND (flags & ".UF_ONLINE.")");
};

sub get_user_id($) {
	my ($user) = @_;
	my ($id, $n);

	if(is_agent($user->{NICK}) and not $enforcers{lc $user->{NICK}}) {
		print "get_user_id called on an agent.\n" if DEBUG;
	}

	eval { $user->{ID} };
	if($@) {
		die("invalid get_user_id call");
	}

	if(exists($user->{ID})) {
		print "The ID of $user->{NICK} is $user->{ID} (from cache)\n" if DEBUG;
		return $user->{ID};
	}

	my $nick = $user->{NICK};

	# a cheat for isServer()
	if($user->{NICK} =~ /\./) {
		return $user->{ID} = undef;
	}

	if($nick eq '') {
		die("get_user_id called on empty string");
	}

	#unless(synced()) {
	#	lock %uidcache;
	#	my $x = $uidcache{lc $nick};
	#	return $x if $x;
	#}
	
	while(($n < 10 or (not synced())) and !defined($id)) {
		$n++;
		$get_user_id->execute($nick);
		($id) = $get_user_id->fetchrow_array;
		unless($id) {
			$get_nickchg->execute($nick);
			($id) = $get_nickchg->fetchrow_array;
		}
	}

	unless($id) { log::wlog(__PACKAGE__, log::DEBUG(), "get_user_id($nick) failed. HDLRS: ". join(' ', @ircd::hdlrs)); }
	
	#unless(synced()) {
	#	lock %uidcache;
	#	$uidcache{lc $nick} = $id;
	#}
	
	print "The ID of $nick is $id\n" if DEBUG;

	return $user->{ID} = $id;
}

sub get_user_nick($) {
	my ($user) = @_;

	eval { $user->{ID} };
	if($@) {
		die("invalid get_user_nick call");
	}
	
	if(exists($user->{NICK})) { return $user->{NICK} }

	$get_user_nick->execute($user->{ID});
	my ($nick) = $get_user_nick->fetchrow_array;

	return $user->{NICK} = $nick;
}

sub is_online($) {
	my ($user) = @_;
	my $nick;

	if(ref($user)) {
		$nick = get_user_nick($user);
	} else {
		$nick = $user;
	}
	
	$is_online->execute($nick);
	my ($status) = $is_online->fetchrow_array;
	
	return $status;
}

sub chk_online($$) {
	my ($user, $target) = @_;

	unless(is_online($target)) {
		if(ref($target)) {
			$target = get_user_nick($target);
		}
		
		notice($user, "\002$target\002: No such user.");
		return 0;
	}
	
	return 1;
}

1;
